
-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Miscellaneous Clutter")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:UpdateKey()
	self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateState")
	self:RegisterMessage("SIXUI_CLUTTER_KEY_CHANGED", "UpdateState")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE", "Broadcast")
	self:UpdateState()
end

function M:OnDisable()
	self:HideClutter()
end

-- Update the clutter key.
function M:UpdateKey()
	if C.CLUTTER_KEY ~= self.config.modifierKey then
		C.CLUTTER_KEY = self.config.modifierKey
		C.CLUTTER_KEY_FUNCTION = C.Enum.ModifierKeyFunction[C.CLUTTER_KEY]
		C.CLUTTER_KEY_STATE_TEXT = C.Enum.ModifierKeyStateText[C.CLUTTER_KEY]
		self:SendMessage("SIXUI_CLUTTER_KEY_CHANGED")
	end
end

-- Update the clutter state.
function M:UpdateState(event, key, state)
	if not self.state and C.CLUTTER_KEY_FUNCTION() then
		self:ShowClutter()
	elseif self.state and not C.CLUTTER_KEY_FUNCTION() then
		self:HideClutter()
	end
end

-- Set the clutter state to shown.
function M:ShowClutter()
	self.state = true
	self:ScheduleRepeatingTimer("UpdateState", 0.1)
	self:Broadcast()
end

-- Set the clutter state to hidden.
function M:HideClutter()
	self.state = false
	self:CancelAllTimers()
	self:Broadcast()
end

-- Broadcast the current clutter state.
function M:Broadcast()
	self:SendMessage("SIXUI_CLUTTER_STATE_CHANGED", self.state)
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:SIXUI_CONFIG_UPDATE(event, module, key, value)
	if module == self:GetName() then
		self:UpdateKey()
	end
end
