
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: coroutine, FRIEND_TAB_RAID, EJMicroButton, ipairs, ToggleFriendsFrame

local C_ChallengeMode = C_ChallengeMode
local GetInstanceInfo = GetInstanceInfo
local RequestRaidInfo = RequestRaidInfo

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display InstanceInfo")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI Instance Info", {
	description = "Shows instance name and bosses while in an instance: green (alive), yellow (alive but saved), red (dead), gray (dead but unsaved).",
	type = "launcher",
	text = "",
	label = "Instance Info",
	glyph = C.Enum.Glyph.InfoCircle,
	visible = false,
	revision = 0,
})

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:RegisterEvent("ENCOUNTER_END", "Update")
	self:RegisterEvent("PLAYER_DIFFICULTY_CHANGED", "Update")
	self:RegisterEvent("SCENARIO_CRITERIA_UPDATE", "Update")
	self:RegisterEvent("UPDATE_INSTANCE_INFO", "Update")
	self:Update()
end

function M:IterateBosses()
	if not self.bosses then
		self.bosses = A.GetJournalBosses(self.journalInstanceID) or {}
	end

	return coroutine.wrap(function()
		for _, boss in ipairs(self.bosses) do
			coroutine.yield(boss)
		end
	end)
end

function M:Update(event)
	local name, type, _, difficultyName = GetInstanceInfo()

	if type == "none" then
		self.bosses = nil
		self.journalInstanceID = nil
		O.label = "Instance Info"
		O.visible = false
	elseif event == "ENCOUNTER_END" then
		RequestRaidInfo()
	else
		self.bosses = nil
		self.journalInstanceID = A.GetJournalInstanceID() or self.journalInstanceID
		O.label = A.FormatInstanceName(name, difficultyName, C_ChallengeMode.GetActiveKeystoneInfo())
		O.visible = true
	end

	O.revision = O.revision + 1
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnClick(frame, button)
	if button == "LeftButton" then
		EJMicroButton:Click()
	else
		ToggleFriendsFrame(FRIEND_TAB_RAID)
	end
end

function O.OnTooltipShow(tooltip)
	tooltip:AddLine(O.label)

	for boss in M:IterateBosses() do
		tooltip:AddLine(C.Enum.BossStatusColor[boss.status]:WrapTextInColorCode(boss.name))
	end

	if not tooltip:IsShown() then
		RequestRaidInfo()
	end
end
